@@ -10,9 +10,11 @@ The interface consists of the following functions:
   compile_element    - Compilation of finite elements
   jit                - Just-In-Time compilation of forms and elements
   default_parameters - Default parameter values for FFC
+  ufc_signature      - Signature of UFC interface (SHA-1 hash of ufc.h)
 """
 
-__version__ = "1.6.0"
+__version__ = "2016.1.0"
+from ffc.git_commit_hash import git_commit_hash
 
 # Import compiler functions
 from ffc.compiler import compile_form, compile_element
@@ -20,8 +22,14 @@ from ffc.compiler import compile_form, compile_element
 # Import JIT compiler
 from ffc.jitcompiler import jit
 
+# Import UFC signature
+from ffc.ufc_signature import ufc_signature
+
+# Import UFC include dir function
+from ffc.ufc_include import get_ufc_include
+
 # Import default parameters
-from .parameters import default_parameters
+from .parameters import default_parameters, default_jit_parameters
 
 # Import plotting
 from .plot import *
