@@ -28,6 +28,7 @@ import numpy
 # UFL modules
 from ufl.utils.sorting import sorted_by_key
 from ufl.utils.derivativetuples import compute_derivative_tuples
+from ufl import custom_integral_types
 
 # FFC modules
 from ffc.log import info, debug, ffc_assert, error, warning
@@ -99,7 +100,8 @@ def _tabulate_tensor(ir, prefix, parameters):
         tensor_code, mem_code, num_ops = _generate_element_tensor(integrals,
                                                                   sets,
                                                                   opt_par,
-                                                                  gdim)
+                                                                  gdim,
+                                                                  tdim)
         tensor_code = "\n".join(tensor_code)
 
         # Set operations equal to num_ops (for printing info on operations).
@@ -125,7 +127,7 @@ def _tabulate_tensor(ir, prefix, parameters):
         cases = [None for i in range(num_facets)]
         for i in range(num_facets):
             # Update transformer with facets and generate case code + set of used geometry terms.
-            c, mem_code, ops = _generate_element_tensor(integrals[i], sets, opt_par, gdim)
+            c, mem_code, ops = _generate_element_tensor(integrals[i], sets, opt_par, gdim, tdim)
             case = [f_comment("Total number of operations to compute element tensor (from this point): %d" % ops)]
             case += c
             cases[i] = "\n".join(case)
@@ -168,7 +170,8 @@ def _tabulate_tensor(ir, prefix, parameters):
                 c, mem_code, ops = _generate_element_tensor(integrals[i][j],
                                                             sets,
                                                             opt_par,
-                                                            gdim)
+                                                            gdim,
+                                                            tdim)
                 case = [f_comment("Total number of operations to compute element tensor (from this point): %d" % ops)]
                 case += c
                 cases[i][j] = "\n".join(case)
@@ -209,7 +212,8 @@ def _tabulate_tensor(ir, prefix, parameters):
             c, mem_code, ops = _generate_element_tensor(integrals[i],
                                                         sets,
                                                         opt_par,
-                                                        gdim)
+                                                        gdim,
+                                                        tdim)
             case = [f_comment("Total number of operations to compute element tensor (from this point): %d" % ops)]
             case += c
             cases[i] = "\n".join(case)
@@ -230,7 +234,16 @@ def _tabulate_tensor(ir, prefix, parameters):
         jacobi_code += "\n"
         jacobi_code += "\n\n" + format["facet determinant"](tdim, gdim) # FIXME: This is not defined in a point???
 
-    elif integral_type == "custom":
+    elif integral_type in custom_integral_types:
+
+        # Set number of cells
+        if integral_type == "cutcell":
+            num_cells = 1
+        elif integral_type == "interface":
+            num_cells = 2
+        elif integral_type == "overlap":
+            num_cells = 2
+        #else: num_cells = ir["num_cells"] at the top
 
         # Warning that more than two cells in only partly supported.
         # The missing piece is to couple multiple cells to
@@ -250,6 +263,7 @@ def _tabulate_tensor(ir, prefix, parameters):
                                                                   sets,
                                                                   opt_par,
                                                                   gdim,
+                                                                  tdim,
                                                                   generate_custom_facet_normal)
 
         tensor_code = "\n".join(tensor_code)
@@ -292,7 +306,7 @@ def _tabulate_tensor(ir, prefix, parameters):
     # restructure this function.
 
     # Add common code except for custom integrals
-    if integral_type != "custom":
+    if integral_type not in custom_integral_types:
         common += _tabulate_weights([quadrature_weights[p] for p in sorted(used_weights)])
 
         # Add common code for updating tables
@@ -345,7 +359,7 @@ def _tabulate_tensor(ir, prefix, parameters):
 
     return "\n".join(common) + "\n" + tensor_code
 
-def _generate_element_tensor(integrals, sets, optimise_parameters, gdim, generate_custom_facet_normal=False):
+def _generate_element_tensor(integrals, sets, optimise_parameters, gdim, tdim, generate_custom_facet_normal=False):
     "Construct quadrature code for element tensors."
 
     # Prefetch formats to speed up code generation.
@@ -354,7 +368,7 @@ def _generate_element_tensor(integrals, sets, optimise_parameters, gdim, generat
     f_I         = format["ip constant"]
     f_loop      = format["generate loop"]
     f_ip_coords = format["generate ip coordinates"]
-    f_coords    = format["vertex_coordinates"]
+    f_coords    = format["coordinate_dofs"]
     f_double    = format["float declaration"]
     f_decl      = format["declaration"]
     f_X         = format["ip coordinates"]
@@ -385,7 +399,7 @@ def _generate_element_tensor(integrals, sets, optimise_parameters, gdim, generat
             name, gdim, ip, r = coordinate
             element_code += ["", f_comment("Declare array to hold physical coordinate of quadrature point.")]
             element_code += [f_decl(f_double, f_X(points, gdim))]
-            ops, coord_code = f_ip_coords(gdim, points, name, ip, r)
+            ops, coord_code = f_ip_coords(gdim, tdim, points, name, ip, r)
             ip_code += ["", f_comment("Compute physical coordinate of quadrature point, operations: %d." % ops)]
             ip_code += [coord_code]
             num_ops += ops
