@@ -26,6 +26,7 @@ import numpy, itertools, collections
 # UFL modules
 from ufl.classes import Form, Integral
 from ufl.sorting import sorted_expr_sum
+from ufl import custom_integral_types
 
 # FFC modules
 from ffc.log import ffc_assert, info, error, warning
@@ -88,7 +89,7 @@ def compute_integral_ir(itg_data,
                                              ir["optimise_parameters"])
 
     # Transform integrals.
-    cell = itg_data.domain.cell()
+    cell = itg_data.domain.ufl_cell()
     ir["trans_integrals"] = _transform_integrals_by_type(ir, transformer, integrals_dict,
                                                          itg_data.integral_type, cell)
 
@@ -120,12 +121,11 @@ def sort_integrals(integrals, default_quadrature_degree, default_quadrature_rule
 
     # Get domain properties from first integral, assuming all are the same
     integral_type  = integrals[0].integral_type()
-    subdomain_id    = integrals[0].subdomain_id()
-    domain_label = integrals[0].domain().label()
-    domain       = integrals[0].domain() # FIXME: Is this safe? Get as input?
+    subdomain_id   = integrals[0].subdomain_id()
+    domain         = integrals[0].ufl_domain()
     ffc_assert(all(integral_type == itg.integral_type() for itg in integrals),
                "Expecting only integrals of the same type.")
-    ffc_assert(all(domain_label == itg.domain().label() for itg in integrals),
+    ffc_assert(all(domain == itg.ufl_domain() for itg in integrals),
                "Expecting only integrals on the same domain.")
     ffc_assert(all(subdomain_id == itg.subdomain_id() for itg in integrals),
                "Expecting only integrals on the same subdomain.")
@@ -184,7 +184,7 @@ def _transform_integrals_by_type(ir, transformer, integrals_dict, integral_type,
             transformer.update_vertex(i)
             terms[i] = _transform_integrals(transformer, integrals_dict, integral_type)
 
-    elif integral_type == "custom":
+    elif integral_type in custom_integral_types:
 
         # Compute transformed integrals: same as for cell integrals
         info("Transforming custom integral")
