@@ -21,9 +21,7 @@ transformers to translate UFL expressions."""
 # Modified by Martin Alnaes, 2013
 # Modified by Garth N. Wells, 2013
 # Modified by Lizao Li, 2015
-#
-# First added:  2009-10-13
-# Last changed: 2015-03-20
+# Modified by Anders Logg, 2015
 
 # Python modules.
 from six.moves import zip
@@ -31,8 +29,9 @@ from numpy import shape, array
 
 # UFL Classes.
 from ufl.classes import FixedIndex, Index
-from ufl.common import StackDict, Stack, product
+from ufl.utils.stacks import StackDict, Stack
 from ufl.permutation import build_component_numbering
+from ufl import custom_integral_types
 
 # UFL Algorithms.
 from ufl.algorithms import Transformer
@@ -342,7 +341,7 @@ class QuadratureTransformerBase(Transformer):
         key = (o, components, derivatives, self.restriction, self.avg)
         basis = self.argument_cache.get(key, None)
 
-        tdim = self.tdim # FIXME: o.domain().topological_dimension() ???
+        tdim = self.tdim
 
         # FIXME: Why does using a code dict from cache make the expression manipulations blow (MemoryError) up later?
         if basis is None or self.optimise_parameters["optimisation"]:
@@ -395,9 +394,9 @@ class QuadratureTransformerBase(Transformer):
         # Get components
         components = self.component()
 
-        en = derivative_expr.rank()
+        en = len(derivative_expr.ufl_shape)
         cn = len(components)
-        ffc_assert(o.rank() == cn, "Expecting rank of grad expression to match components length.")
+        ffc_assert(len(o.ufl_shape) == cn, "Expecting rank of grad expression to match components length.")
 
         # Get direction of derivative
         if cn == en+1:
@@ -449,7 +448,7 @@ class QuadratureTransformerBase(Transformer):
             ffc_assert(not (derivatives and is_quad_element), \
                        "Derivatives of Quadrature elements are not supported: " + repr(o))
 
-            tdim = self.tdim # FIXME: o.domain().topological_dimension() ???
+            tdim = self.tdim
 
             # Create code for function and add empty tuple to cache dict.
             function_code = {(): self.create_function(o, derivatives, component,
@@ -460,74 +459,6 @@ class QuadratureTransformerBase(Transformer):
 
         return function_code
 
-    def constant(self, o):
-        #print("\n\nVisiting Constant: " + repr(o))
-
-        # Map o to object with proper element and count
-        o = self._function_replace_map[o]
-
-        # Safety checks.
-        ffc_assert(len(self.component()) == 0, "Constant does not expect component indices: " + repr(self._components))
-        ffc_assert(o.shape() == (), "Constant should not have a value shape: " + repr(o.shape()))
-
-        # Component default is 0
-        component = 0
-
-        # Handle restriction.
-        if self.restriction == "-":
-            component += 1
-
-        # Let child class create constant symbol
-        coefficient = format["coefficient"](o.count(), component)
-        return self._create_symbol(coefficient, CONST)
-
-    def vector_constant(self, o):
-        #print("\n\nVisiting VectorConstant: " + repr(o))
-
-        # Map o to object with proper element and count
-        o = self._function_replace_map[o]
-
-        # Get the component
-        components = self.component()
-
-        # Safety checks.
-        ffc_assert(len(components) == 1, "VectorConstant expects 1 component index: " + repr(components))
-
-        # We get one component.
-        component = components[0]
-
-        # Handle restriction.
-        if self.restriction == "-":
-            component += o.shape()[0]
-
-        # Let child class create constant symbol
-        coefficient = format["coefficient"](o.count(), component)
-        return self._create_symbol(coefficient, CONST)
-
-    def tensor_constant(self, o):
-        #print("\n\nVisiting TensorConstant: " + repr(o))
-
-        # Map o to object with proper element and count
-        o = self._function_replace_map[o]
-
-        # Get the components
-        components = self.component()
-
-        # Safety checks.
-        ffc_assert(len(components) == len(o.shape()), \
-                   "The number of components '%s' must be equal to the number of shapes '%s' for TensorConstant." % (repr(components), repr(o.shape())))
-
-        # Let the UFL element handle the component map.
-        component = o.element()._sub_element_mapping[components]
-
-        # Handle restriction (offset by value shape).
-        if self.restriction == "-":
-            component += product(o.shape())
-
-        # Let child class create constant symbol
-        coefficient = format["coefficient"](o.count(), component)
-        return self._create_symbol(coefficient, CONST)
-
     # -------------------------------------------------------------------------
     # SpatialCoordinate (geometry.py).
     # -------------------------------------------------------------------------
@@ -609,50 +540,62 @@ class QuadratureTransformerBase(Transformer):
     # -------------------------------------------------------------------------
     def sqrt(self, o, *operands):
         #print("\n\nVisiting Sqrt: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["sqrt"])
 
     def exp(self, o, *operands):
         #print("\n\nVisiting Exp: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["exp"])
 
     def ln(self, o, *operands):
         #print("\n\nVisiting Ln: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["ln"])
 
     def cos(self, o, *operands):
         #print("\n\nVisiting Cos: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["cos"])
 
     def sin(self, o, *operands):
         #print("\n\nVisiting Sin: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["sin"])
 
     def tan(self, o, *operands):
         #print("\n\nVisiting Tan: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["tan"])
 
     def cosh(self, o, *operands):
         #print("\n\nVisiting Cosh: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["cosh"])
 
     def sinh(self, o, *operands):
         #print("\n\nVisiting Sinh: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["sinh"])
 
     def tanh(self, o, *operands):
         #print("\n\nVisiting Tanh: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["tanh"])
 
     def acos(self, o, *operands):
         #print("\n\nVisiting Acos: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["acos"])
 
     def asin(self, o, *operands):
         #print("\n\nVisiting Asin: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["asin"])
 
     def atan(self, o, *operands):
         #print("\n\nVisiting Atan: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["atan"])
 
     def atan_2(self, o, *operands):
@@ -662,6 +605,7 @@ class QuadratureTransformerBase(Transformer):
 
     def erf(self, o, *operands):
         #print("\n\nVisiting Erf: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
         return self._math_function(operands, format["erf"])
 
     def bessel_i(self, o, *operands):
@@ -925,7 +869,7 @@ class QuadratureTransformerBase(Transformer):
         "Helper function for both Coefficient and Argument."
 
         # Get UFL element.
-        ufl_element = ufl_function.element()
+        ufl_element = ufl_function.ufl_element()
 
         # Get subelement and the relative (flattened) component (in case we have mixed elements).
         local_comp, local_elem = ufl_element.extract_component(component)
@@ -935,10 +879,10 @@ class QuadratureTransformerBase(Transformer):
             # Map component using component map from UFL. (TODO: inefficient use of this function)
             comp_map, _ = build_component_numbering(local_elem.value_shape(), local_elem.symmetry())
             local_comp = comp_map[local_comp]
-            
+
         # Set local_comp to 0 if it is ()
         if not local_comp: local_comp = 0
-           
+
         # Check that component != not () since the UFL component map will turn
         # it into 0, and () does not mean zeroth component in this context.
         if len(component):
@@ -965,7 +909,7 @@ class QuadratureTransformerBase(Transformer):
                    "Assuming subelement mappings are equal but they differ.")
 
         # Generate FFC multi index for derivatives.
-        tdim = self.tdim # FIXME: ufl_element.domain().topological_dimension() ???
+        tdim = self.tdim
         multiindices = FFCMultiIndex([list(range(tdim))]*len(derivatives)).indices
 
         return (component, local_elem, local_comp, local_offset, ffc_element, transformation, multiindices)
@@ -1000,7 +944,7 @@ class QuadratureTransformerBase(Transformer):
                    "Currently, Argument part is not supporte: " + repr(ufl_argument))
 
         # Get element counter and loop index.
-        element_counter = self.element_map[1 if avg else self.points][ufl_argument.element()]
+        element_counter = self.element_map[1 if avg else self.points][ufl_argument.ufl_element()]
         loop_index = indices[ufl_argument.number()]
 
         # Offset element space dimension in case of negative restriction,
@@ -1014,7 +958,7 @@ class QuadratureTransformerBase(Transformer):
 
         # Create basis access, we never need to map the entry in the basis table
         # since we will either loop the entire space dimension or the non-zeros.
-        if self.restriction in ("+", "-") and self.integral_type == "custom" and offset != "":
+        if self.restriction in ("+", "-") and self.integral_type in custom_integral_types and offset != "":
             # Special case access for custom integrals (all basis functions stored in flattened array)
             basis_access = format["component"]("", [f_ip, format["add"]([loop_index, offset])])
         else:
@@ -1029,7 +973,7 @@ class QuadratureTransformerBase(Transformer):
 
         # If domain type is custom, then special-case set loop index
         # range since table is empty
-        if self.integral_type == "custom":
+        if self.integral_type in custom_integral_types:
             loop_index_range = ffc_element.space_dimension() # different from `space_dimension`...
 
         basis = ""
@@ -1077,7 +1021,7 @@ class QuadratureTransformerBase(Transformer):
         f_ip = "0" if (avg or self.points == 1) else format["integration points"]
 
         # Get the element counter.
-        element_counter = self.element_map[1 if avg else self.points][ufl_function.element()]
+        element_counter = self.element_map[1 if avg else self.points][ufl_function.ufl_element()]
 
         # Get current cell entity, with current restriction considered
         entity = self._get_current_entity()
@@ -1099,7 +1043,7 @@ class QuadratureTransformerBase(Transformer):
 
         # If domain type is custom, then special-case set loop index
         # range since table is empty
-        if self.integral_type == "custom":
+        if self.integral_type in custom_integral_types:
             loop_index_range = ffc_element.space_dimension()
 
         # Create loop index
@@ -1203,7 +1147,7 @@ class QuadratureTransformerBase(Transformer):
                 function_count = len(self.function_data)
                 data = (function_count, loop_index_range,
                         self._count_operations(function_expr),
-                        psi_name, used_nzcs, ufl_function.element())
+                        psi_name, used_nzcs, ufl_function.ufl_element())
                 self.function_data[function_expr] = data
             function_symbol_name = format["function value"](data[0])
 
