@@ -17,16 +17,17 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Anders Logg, 2009.
+# Modified by Anders Logg, 2009, 2015
 # Modified by Martin Alnaes, 2013-2014
 
 import numpy, itertools
 
 # UFL modules
 import ufl
-from ufl.cell import Cell
-from ufl.classes import Grad, CellAvg, FacetAvg
+from ufl.cell import Cell, num_cell_entities
+from ufl.classes import ReferenceGrad, Grad, CellAvg, FacetAvg
 from ufl.algorithms import extract_unique_elements, extract_type, extract_elements
+from ufl import custom_integral_types
 
 # FFC modules
 from ffc.log import ffc_assert, info, error, warning
@@ -34,19 +35,7 @@ from ffc.utils import product
 from ffc.fiatinterface import create_element
 from ffc.fiatinterface import map_facet_points, reference_cell_vertices
 from ffc.quadrature_schemes import create_quadrature
-
-def _create_quadrature_points_and_weights(integral_type, cellname, facet_cellname, degree, rule):
-    if integral_type == "cell":
-        (points, weights) = create_quadrature(cellname, degree, rule)
-    elif integral_type == "exterior_facet" or integral_type == "interior_facet":
-        (points, weights) = create_quadrature(facet_cellname, degree, rule)
-    elif integral_type == "vertex":
-        (points, weights) = ([()], numpy.array([1.0,])) # TODO: Will be fixed
-    elif integral_type == "custom":
-        (points, weights) = (None, None)
-    else:
-        error("Unknown integral type: " + str(integral_type))
-    return (points, weights)
+from ffc.representationutils import create_quadrature_points_and_weights
 
 def _find_element_derivatives(expr, elements, element_replace_map):
     "Find the highest derivatives of given elements in expression."
@@ -58,7 +47,7 @@ def _find_element_derivatives(expr, elements, element_replace_map):
     num_derivatives = dict((e, 0) for e in elements)
 
     # Extract the derivatives from the integral.
-    derivatives = set(extract_type(expr, Grad))
+    derivatives = set(extract_type(expr, Grad)) | set(extract_type(expr, ReferenceGrad))
 
     # Loop derivatives and extract multiple derivatives.
     for d in list(derivatives):
@@ -67,7 +56,7 @@ def _find_element_derivatives(expr, elements, element_replace_map):
         elem, = extract_elements(d.ufl_operands[0])
         elem = element_replace_map[elem]
         # Set the number of derivatives to the highest value encountered so far.
-        num_derivatives[elem] = max(num_derivatives[elem], len(extract_type(d, Grad)))
+        num_derivatives[elem] = max(num_derivatives[elem], len(extract_type(d, Grad)), len(extract_type(d, ReferenceGrad)))
     return num_derivatives
 
 def domain_to_entity_dim(integral_type, tdim):
@@ -77,7 +66,7 @@ def domain_to_entity_dim(integral_type, tdim):
         entity_dim = tdim - 1
     elif integral_type == "vertex":
         entity_dim = 0
-    elif integral_type == "custom":
+    elif integral_type in custom_integral_types:
         entity_dim = tdim
     else:
         error("Unknown integral_type: %s" % integral_type)
@@ -120,10 +109,9 @@ def _tabulate_psi_table(integral_type, cellname, tdim, element, deriv_order, poi
     # Handle case when list of points is empty
     if points is None:
         return _tabulate_empty_psi_table(tdim, deriv_order, element)
-
     # Otherwise, call FIAT to tabulate
     entity_dim = domain_to_entity_dim(integral_type, tdim)
-    num_entities = Cell(cellname).num_entities(entity_dim)
+    num_entities = num_cell_entities[cellname][entity_dim]
     psi_table = {}
     for entity in range(num_entities):
         entity_points = _map_entity_points(cellname, tdim, points, entity_dim, entity)
@@ -138,7 +126,7 @@ def _tabulate_entities(integral_type, cellname, tdim):
     # MSA: I attempted to generalize this function, could this way of
     # handling domain types generically extend to other parts of the code?
     entity_dim = domain_to_entity_dim(integral_type, tdim)
-    num_entities = Cell(cellname).num_entities(entity_dim)
+    num_entities = num_cell_entities[cellname][entity_dim]
     entities = set()
     for entity in range(num_entities):
         # TODO: Use 0 as key for cell and we may be able to generalize other places:
@@ -172,19 +160,22 @@ def tabulate_basis(sorted_integrals, form_data, itg_data):
     integrals = {}
     avg_elements = { "cell": [], "facet": [] }
 
+    # Get some useful variables in short form
     integral_type = itg_data.integral_type
-    cellname = itg_data.domain.cell().cellname()
-    facet_cellname = itg_data.domain.cell().facet_cellname()
+    cell = itg_data.domain.ufl_cell()
+    cellname = cell.cellname()
     tdim = itg_data.domain.topological_dimension()
 
-    # Loop the quadrature points and tabulate the basis values.
+    # Create canonical ordering of quadrature rules
     rules = sorted(sorted_integrals.keys())
+
+    # Loop the quadrature points and tabulate the basis values.
     for degree, scheme in rules:
 
         # --------- Creating quadrature rule
         # Make quadrature rule and get points and weights.
-        (points, weights) = _create_quadrature_points_and_weights(integral_type, cellname,
-                                                                  facet_cellname, degree, scheme)
+        (points, weights) = create_quadrature_points_and_weights(integral_type, cell, degree, scheme)
+
         # The TOTAL number of weights/points
         len_weights = None if weights is None else len(weights)
 
@@ -195,7 +186,6 @@ def tabulate_basis(sorted_integrals, form_data, itg_data):
 
 
         # --------- Store integral
-
         # Add the integral with the number of points as a key to the return integrals.
         integral = sorted_integrals[(degree, scheme)]
         ffc_assert(len_weights not in integrals, \
@@ -210,12 +200,13 @@ def tabulate_basis(sorted_integrals, form_data, itg_data):
                         for e in extract_unique_elements(integral)]
 
         # Insert elements for x and J
-        domain = integral.domain() # FIXME: For all domains to be sure? Better to rewrite though.
-        x_element = domain.coordinate_element()
+        domain = integral.ufl_domain() # FIXME: For all domains to be sure? Better to rewrite though.
+        x_element = domain.ufl_coordinate_element()
         if x_element not in ufl_elements:
-            if integral_type == "custom":
+            if integral_type in custom_integral_types:
                 # FIXME: Not yet implemented, in progress
-                warning("Vector elements not yet supported in custom integrals so element for coordinate function x will not be generated.")
+                #warning("Vector elements not yet supported in custom integrals so element for coordinate function x will not be generated.")
+                pass
             else:
                 ufl_elements.append(x_element)
 
@@ -265,8 +256,7 @@ def tabulate_basis(sorted_integrals, form_data, itg_data):
             fiat_element = create_element(element)
 
             # Make quadrature rule and get points and weights.
-            (points, weights) = _create_quadrature_points_and_weights(avg_integral_type, cellname,
-                                                                      facet_cellname, element.degree(), "default")
+            (points, weights) = create_quadrature_points_and_weights(avg_integral_type, cell, element.degree(), "default")
             wsum = sum(weights)
 
             # Tabulate table of basis functions and derivatives in points
