@@ -27,7 +27,7 @@ import numpy
 from FIAT.functional import PointEvaluation
 
 # FFC modules.
-from log import error, info_red
+from .log import error, info_red
 
 # Default quadrature element degree
 default_quadrature_degree = 1
@@ -51,9 +51,8 @@ class QuadratureElement:
         # Create quadrature (only interested in points)
         # TODO: KBO: What should we do about quadrature functions that live on ds, dS?
         # Get cell and facet names.
-        domain, = ufl_element.domains() # Assuming single domain
-        cellname = domain.cell().cellname()
-        #facet_cellname = domain.cell().facet_cellname()
+        cell = ufl_element.cell()
+        cellname = cell.cellname()
         points, weights = create_quadrature(cellname, degree, self._quad_scheme)
 
         # Save the quadrature points
@@ -70,7 +69,7 @@ class QuadratureElement:
         # Save the geometric dimension.
         # FIXME: KBO: Do we need to change this in order to integrate on facets?
         #        MSA: Not the geometric dimension, but maybe the topological dimension somewhere?
-        self._geometric_dimension = domain.geometric_dimension()
+        self._geometric_dimension = cell.geometric_dimension()
 
     def space_dimension(self):
         "The element space dimension is simply the number of quadrature points"
@@ -114,8 +113,8 @@ class QuadratureElement:
 
         # Check that incoming points are equal to the quadrature points.
         if len(points) != len(self._points) or abs(numpy.array(points) - self._points).max() > 1e-12:
-            print "\npoints:\n", numpy.array(points)
-            print "\nquad points:\n", self._points
+            print("\npoints:\n", numpy.array(points))
+            print("\nquad points:\n", self._points)
             error("Points must be equal to coordinates of quadrature points")
 
         # Return the identity matrix of size len(self._points) in a
@@ -131,7 +130,7 @@ def _create_entity_dofs(fiat_cell, num_dofs):
         entity_dofs[dim] = {}
         for entity in sorted( top[dim] ):
             entity_dofs[dim][entity]=[]
-    entity_dofs[dim][0] = range(num_dofs)
+    entity_dofs[dim][0] = list(range(num_dofs))
     return entity_dofs
 
 # FFC modules to avoid circular import
